using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using XNATutorial.Entity;
using XNATutorial.Camera;
using XNATutorial.Camera.Interfaces;
using XNATutorial.Input;

namespace XNATutorial
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game : Microsoft.Xna.Framework.Game
    {
        private GraphicsDeviceManager Graphics { get; set; }

        private ICamera Camera { get; set; }
        private CameraInputComponent CameraInputComponent { get; set; }

        private IEntity SphereEntity { get; set; }
        private EntityInputComponent EntityInputComponent { get; set; }

        public Game()
        {
            Window.Title = "XNA 4.0 Tutorial";

            Content.RootDirectory = "Content";

            Graphics = new GraphicsDeviceManager(this);

            Graphics.PreferMultiSampling = true; // Turn on antialiasing 
            Graphics.SynchronizeWithVerticalRetrace = true; // Turn on VSync
            Graphics.GraphicsProfile = GraphicsProfile.HiDef; // Turn on best graphic settings

            Graphics.IsFullScreen = false;

            if (Graphics.IsFullScreen) // fullscreen
            {
                Graphics.PreferredBackBufferWidth = 1920;
                Graphics.PreferredBackBufferHeight = 1200;

                Window.AllowUserResizing = false;
            }
            else // window mode
            {
                Graphics.PreferredBackBufferWidth = 1280;
                Graphics.PreferredBackBufferHeight = 720;

                Window.AllowUserResizing = true;
            }

            Graphics.ApplyChanges();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Initialize camera
            Camera = new QuaternionCamera(MathHelper.ToRadians(45.0f), GraphicsDevice.Viewport.AspectRatio, 1.0f, 1000.0f);

            // Initialize input components
            CameraInputComponent = new CameraInputComponent(this);
            CameraInputComponent.Entity = Camera;
            Components.Add(CameraInputComponent);

            EntityInputComponent = new EntityInputComponent(this);
            Components.Add(EntityInputComponent);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            var effect = new BasicEffect(GraphicsDevice);
            effect.EnableDefaultLighting();
            effect.AmbientLightColor = Color.Blue.ToVector3();
            var sphereModel = Content.Load<Model>("models/sphere");
            SphereEntity = new ModelEntity(effect, sphereModel);

            EntityInputComponent.Entity = SphereEntity; // Enables user to move the sphere
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            SphereEntity.Rotate(Vector3.UnitY, gameTime.ElapsedGameTime.Milliseconds * 0.01f);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            SphereEntity.Draw(gameTime, Camera.View, Camera.Projection);

            base.Draw(gameTime);
        }
    }
}
