using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNATutorial.Entity;

namespace XNATutorial.Input
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class CameraInputComponent : GameComponent
    {
        public IMoveableEntity Entity { get; set; }

        private KeyboardState currentKeyboardState;
        private KeyboardState previousKeyboardState;
        
        public CameraInputComponent(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            currentKeyboardState = Keyboard.GetState();

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            previousKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState();

            var translationStep = 0.1f * gameTime.ElapsedGameTime.Milliseconds;

            if (currentKeyboardState.IsKeyDown(Keys.W))
            {
                Entity.Translate(Vector3.Forward, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.S))
            {
                Entity.Translate(Vector3.Backward, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.A))
            {
                Entity.Translate(Vector3.Left, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.D))
            {
                Entity.Translate(Vector3.Right, translationStep);
            }

            var rotationStep = 0.1f * gameTime.ElapsedGameTime.Milliseconds;

            if (currentKeyboardState.IsKeyDown(Keys.Q))
            {
                Entity.Rotate(Vector3.UnitZ, rotationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.E))
            {
                Entity.Rotate(Vector3.UnitZ, -rotationStep);
            }

            base.Update(gameTime);
        }
    }
}