﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using XNATutorial.Entity;
using Microsoft.Xna.Framework.Input;

namespace XNATutorial.Input
{
    class EntityInputComponent : GameComponent
    {
        public IMoveableEntity Entity { get; set; }

        private KeyboardState currentKeyboardState;
        private KeyboardState previousKeyboardState;

        public EntityInputComponent(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            currentKeyboardState = Keyboard.GetState();

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            previousKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState();

            var translationStep = 0.1f * gameTime.ElapsedGameTime.Milliseconds;

            if (currentKeyboardState.IsKeyDown(Keys.Up))
            {
                Entity.Translate(Vector3.Forward, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Down))
            {
                Entity.Translate(Vector3.Backward, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Left))
            {
                Entity.Translate(Vector3.Left, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Right))
            {
                Entity.Translate(Vector3.Right, translationStep);
            }

            base.Update(gameTime);
        }
    }
}
